
; pokersol.asm

; by B. Watson <atari@hardcoders.org>

; Idea for a silly game...

; (not yet) Tested on a real Atari 2600
; Tested on z26 v1.46 on a windows machine
; Tested on xstella 1.1 on a linux machine

 processor 6502
 include "vcs.h"

 seg.u data

 org $80

card0ptr ds 50
left_diff_ptr = card0ptr
right_diff_ptr = card0ptr+2
ranks = card0ptr
suits = card0ptr+14
pairs = suits+5
trips = pairs+1
quad  = trips+1
flush = quad+1
tmp_cards = flush+1 ; 5 bytes
end_ptrs
tableau ds 25 ; the cards that have been played
stock ds 27 ; the rest of the cards
scoredig0 = stock ; 3 2-byte pointers
deck = tableau ; the whole deck (which gets shuffled, then the
               ; first 25 cards get overwritten with the
               ; blank tableau)
current_card ds 1 ; index into stock, points at card being placed
tmp ds 1
tmp2 ds 1
tmp3 ds 1
playerpos ds 1 ; 5*y + x (or else $FF for `game not on'
framectr ds 1
color0 ds 5
random ds 2
score ds 1
already_scored ds 1
score_frame ds 1
sound_type ds 1
sound_ctr ds 1
btn_debnc ds 1
;last_btn ds 1
joy_debnc ds 1
last_joy ds 1
bw_mask ds 1 ; $FF for color, $0F for B/W (ANDed with color values)
diff_shadow ds 1 ; shadows SWCHB

 echo *-$80,"bytes of zero page used."

	if (*-$80) > ($7d)
	echo "You're out of zero page! (Must be $7D or less bytes used)"
	err
	endif
 

; constants:
RED = $34 ; was $32, looks great in color, crappy in B/W
CARD_COLOR = $0C ; white (don't need to bother with bw_mask for this)
YELLOW = $FC
GREEN = $B6

SND_BZZT = $8A ; hi nyb = D4-D1 of AUDF0, lo nyb = D3-D0 of AUDC0
SND_BZZT_LEN = $06 ; frames to play sound for

;these are OK
SND_PLACE = $22
SND_PLACE_LEN = $03

SND_SCORE = $1A
SND_SCORE_LEN = $20

 seg code

 org $F000

main_loop
 lda #2
 sta VSYNC ; start blanking
 sta WSYNC
 sta WSYNC
 lda #44
 sta TIM64T ; go ahead & set timer
 lda #0
 sta WSYNC
 sta VSYNC ; 3 WSYNC's, then turn off VSYNC

game_calc
 inc framectr

; position players
 sta WSYNC
; repeat 23
; nop
; repend
 ldx #9
posloop
 dex
 bne posloop
 sta RESP0 ; 49
 sta RESP1 ; 52
 lda #$d0
 sta HMP1
 lda #$40
 sta HMP0
 sta WSYNC
 sta HMOVE
; repeat 12
; nop
; repend
 
 lda #CARD_COLOR ; 2
 sta COLUP0 ; 3 = 5
 sta COLUP1 ; 3 = 8
 ldy #0 ; 2 = 12
 repeat 6 ; DAMMIT do I need these NOPs??
 nop
 repend
 sta HMCLR

; setup pointers to card sprite data (25 of them!)
;   note to self: saved 50 cycles by swapping roles of X and Y here
;   (I had 2 sta zp,y's, which are actually sta.w, since there's no sta zp,y)
;   there is still an lda zp,y (same deal), but not much I can do about this
;   put it in stelladoc as an example...

; these are already 0 from above
;ldy #0
;ldx #0
card_disp
 sty tmp
 lda tableau,y
 tay
 lda lo_bytes,y
 sta card0ptr,x
 inx
 lda hi_bytes,y
 sta card0ptr,x
 inx
 ldy tmp
 iny
 cpy #25
 bne card_disp

 ldy #$0F
 lda SWCHB
 and #8
 beq bw_on
 ldy #$FF
bw_on
 sty bw_mask 

; set up card color bytes (1 per row)
 ldx #0
 stx color0
 stx color0+1
 stx color0+2
 stx color0+3
 stx color0+4
cbyte_loop
 lda tableau,x
 cmp #26
 rol color0
 lda tableau+5,x
 cmp #26
 rol color0+1
 lda tableau+10,x
 cmp #26
 rol color0+2
 lda tableau+15,x
 cmp #26
 rol color0+3
 lda tableau+20,x
 cmp #26
 rol color0+4
 inx
 cpx #5
 bne cbyte_loop

 
; blink cursor 
 lda framectr
 and #8 ; blink 8 frames, then don't blink 8 frames
 beq no_blink 
 lda current_card
 cmp #25
 beq no_blink ; dont blink if game is over
 lda playerpos
 asl
 tax
 lda #<no_card
 sta card0ptr,x
 lda #>no_card
 sta card0ptr+1,x

no_blink
; lda framectr
; and #7
; beq do_stick
; jmp no_stick

do_stick
 ;sta last_joy
 lda INPT4
 and #$80
 ;cmp #$80
 bne not_hard
 ;cmp last_btn
 ;bne no_btn_debnc
 lda btn_debnc
 beq no_btn_debnc
 dec btn_debnc
 bne not_hard
 ;bmi no_fire
no_btn_debnc
 ;sta last_btn
 lda #16
 sta btn_debnc
 lda playerpos ; check & see if games is already on ($FF = no)
 bpl place_card
 lda #0 ; player pressed fire during shuffling
 sta playerpos ; so start the game
 sta current_card
 ldx #24
 lda #52 ; blank card
blank_tableau
 sta tableau,x
 dex
 bpl blank_tableau
 ;bmi no_fire ; always branch
 ; now the game is ready to begin, let's check left diff. switch
 lda diff_shadow
 and #64
 beq not_hard
 lda stock      ; if left `pro' difficulty, deal the 1st 5 cards to the
 sta tableau+12 ; corners and center. *Damn* this makes the game harder!
 lda stock+1    ; (thanks, Glenn!)
 sta tableau
 lda stock+2
 sta tableau+4
 lda stock+3
 sta tableau+20
 lda stock+4
 sta tableau+24
 lda #5
 sta current_card
not_hard
 jmp no_fire
; ;DEBUG CODE
; ldx #5
; stx current_card
; dex
;stack_deck
; lda stacked,x
; sta tableau,x
; dex
; bpl stack_deck
; ;END DEBUG CODE


place_card ; player pressed fire while game is on
 ldx current_card
 ldy playerpos
 lda #SND_BZZT
 sta sound_type
 lda #SND_BZZT_LEN
 sta sound_ctr
 lda tableau,y ; can't place a card where a card already is
 cmp #52 ; 52 is `no card placed'
 bne no_fire
 lda stock,x
 sta tableau,y
 inc current_card
 lda #SND_PLACE
 sta sound_type
 lda #SND_PLACE_LEN
 sta sound_ctr
no_fire

 lda playerpos
; bmi no_stick
 bpl check_stick
 lda SWCHB
 and #$40
 sta diff_shadow
 jmp no_stick

; read joystick

check_stick
 lda SWCHA
 and #$F0
 cmp #$F0
 beq no_stick
 cmp last_joy
 bne no_joy_debnc
 dec joy_debnc
 bpl no_stick
no_joy_debnc
 sta last_joy
 ldx #8
 stx joy_debnc
 asl
 bcs no_right
 inc playerpos
no_right
 asl
 bcs no_left
 dec playerpos
no_left
 asl
 bcs no_down
 tax
 lda playerpos
 adc #5
 sta playerpos
 txa
no_down
 asl
 bcs no_up 
 lda playerpos
 sbc #4 ; subtract 5 (carry always clear)
 sta playerpos
no_up
 lda playerpos
 bpl not_minus
 clc
 adc #25
 sta playerpos
not_minus
 cmp #25
 bcc not_toohigh
 sec
 sbc #25
 sta playerpos
not_toohigh

no_stick

 lda SWCHB
 and #$01
 bne no_reset
 jsr deal_deck ; player pressed game reset
 lda #$FF ; $FF in playerpos means `game not on, so shuffle'
 sta playerpos

no_reset

decode_score
 lda current_card
 cmp #25
 bne no_decode_score
 lda #0
 sta tmp
 sta tmp2
 sta tmp3
 lda #<score_blank
 sta scoredig0
 sta scoredig0+2
; sta scoredig0+4
 lda #>score_blank
 sta scoredig0+1
 sta scoredig0+3
 sta scoredig0+5
 lda score
ck_hund
 cmp #100
 bcc hundreds_done
 inc tmp
 sec
 sbc #100
 bcs ck_hund
; adc #100
hundreds_done
ck_tens
 cmp #10
 bcc tens_done
 inc tmp2
 sec
 sbc #10
 bcs ck_tens
; adc #10
tens_done
 asl
 asl
 asl
 sta scoredig0+4 ; ones always displayed, even if 0
 lda tmp2
 beq no_tens ; leave tens blank if score < 10
 asl
 asl
 asl
 sta scoredig0+2
no_tens
 lda tmp
 beq no_hund ; leave hunds blank if score < 100
 asl
 asl
 asl
 sta scoredig0 
no_hund

no_decode_score

 lda #0
 sta COLUBK
 sta COLUPF
 sta PF0
 sta PF1
 sta PF2

; play sounds
 lda sound_ctr
 bne play_sound
; lda #$00
 sta AUDV0
 sta AUDF0
 beq sound_done

play_sound
 lda sound_ctr
 and #$07
 sta AUDV0
 lda sound_type
 tax
 and #$F0
 lsr
 lsr
 lsr
 sta AUDF0
 stx AUDC0
 dec sound_ctr

sound_done

wait_timer
 lda INTIM
 bne wait_timer ; busy-wait for timer to expire
 sta WSYNC
 sta VBLANK
 sta WSYNC ; wby do I need these 2??
 lda #6
 sta NUSIZ0
 lda #2
 sta NUSIZ1
 sta WSYNC

; check Select switch, show instructions if pressed
 lda SWCHB
 and #2
 bne not_sel
 jmp inst_kernel
not_sel

kernel
 ;lda #CARD_COLOR
 ;sta COLUP0
 ;ldx #192

 mac cardloop
 lda #0
 sta COLUBK
 lda #%01111110
 sta GRP0
 sta GRP1
 sta WSYNC
 lda #%11111111
 sta GRP0
 sta GRP1
 sta PF0
 sta PF1
 sta PF2
 ldx color0+{1} ; 3
 lda colPF0,x ; +4 = 7
 sta PF0 ; +3 = 11
 lda colPF1,x ; +4 = 15
 sta PF1 ; +3 = 18
 lda colPF2,x ; +4 = 22
 sta PF2 ; +3 = 25

 ldy #11
.cardloop0
 repeat 2
 sta WSYNC ; 0
 lda #0
 sta.w COLUBK
 lda (card0ptr+{1}*10+0),y ; 5
 sta GRP0 ; 3 = 8
 lda (card0ptr+{1}*10+2),y ; 5 = 13
 sta GRP1 ; 3 = 16
 lda (card0ptr+{1}*10+8),y ; 5 = 21
 tax ; 2 = 23
 txs ; 2 = 25
 lda (card0ptr+{1}*10+6),y ; 5 = 30
 tax ; 2 = 32

; before color/bw code...
; nop
; nop
; lda #RED
; sta.w COLUBK ; 42
;
 nop
 lda #RED
 and bw_mask
 sta COLUBK ; 42

 lda (card0ptr+{1}*10+4),y ; 5 = 47
 sta GRP0 ; 3 = 50
 stx GRP1 ; 3 = 53
 tsx ; 2 = 55
 stx GRP0 ; 3 = 58
 repend 
 dey ; 2 = 60
 bpl .cardloop0

 sta WSYNC
 lda #%11111111
 sta GRP0
 sta GRP1
 sta PF0
 sta PF1
 sta PF2
 sta WSYNC
 lda #%01111110
 sta GRP0
 sta GRP1
 sta WSYNC
 lda #0
 sta GRP0
 sta GRP1

 sta WSYNC
 iny
 sty GRP0
 sty GRP1
 sta WSYNC
 endm ; end of mac cardloop

 cardloop 0
 cardloop 1
 cardloop 2
 cardloop 3
 cardloop 4

 sta WSYNC
 ldx #31
 lda playerpos
; bmi kill_rest ; this branch is too far!
 bpl k_ok
 jmp kill_rest
k_ok
 lda current_card
 ;jmp draw_score ; very temporary!
 cmp #25
; beq kill_rest ; this branch is too far!
 bne k_ok2
 jmp draw_score
k_ok2
 sta WSYNC

 lda #0
 sta NUSIZ0
 lda #%01111110
 sta GRP0
 sta WSYNC
 lda #%11111111
 sta GRP0
 ldx current_card
 lda stock,x
 tax
 lda lo_bytes,x
 sta tmp
 lda hi_bytes,x
 sta tmp+1
 lda #0
 cpx #26
 ldx #$F0
 bcc last_card_black
 ldx #$c0
last_card_black
 ldy #11
draw_last_card
 repeat 2
 sta WSYNC
 lda #0
 sta COLUBK
 lda (tmp),y
 sta GRP0
 stx PF0

;;; waste 16 cycles (with sta.w below)
; repeat 8 ; sketchy... 2*8=16 bytes total
; nop
; repend

; waste 16 cycles, without the sta.w below
 lda (tmp),y ; this is 7*2=14 bytes total, plus we lose the sta.w below
 lda (tmp),y
 lda (tmp),y
 nop

 lda #RED
 and bw_mask
 sta COLUBK ; sta.w COLUBK
 repend
 dey
 bpl draw_last_card
; nop
; nop
; nop

 sta WSYNC 
 lda #0
 sta COLUBK
 lda #%11111111
 sta GRP0
 sta WSYNC 
 lda #%01111110
 sta GRP0
 sta WSYNC
 lda #0
 sta GRP0 


 ldx #2
kill_rest
 sta WSYNC
 dex
 bne kill_rest

 lda diff_shadow
 bne ld_on
 lda #<no_card 
 sta  left_diff_ptr
 lda #>no_card
 sta  left_diff_ptr+1
 jmp ld_off
ld_on
 lda #<left_diff_hard 
 sta  left_diff_ptr
 lda #>left_diff_hard
 sta  left_diff_ptr+1
ld_off

 lda #0
 sta NUSIZ0
; sta NUSIZ1
 ldy #7
kr_diff_loop
 sta WSYNC
; lda (right_diff_ptr),y
; sta GRP1
 lda (left_diff_ptr),y
 sta GRP0
 dey
 bpl kr_diff_loop
 sta WSYNC
; lda #0
; sta GRP0
; sta GRP1
 jmp end_kernel
 
draw_score
 lda #2
 sta NUSIZ0
 lda #0
 sta NUSIZ1
 ldy #7
 lda score
 cmp #24
 bcc le_24
 cmp #39
 bcc le_39
 lda #GREEN
 bne ds_color
le_24
 lda #RED
 bne ds_color
le_39
 lda #YELLOW

ds_color
 and bw_mask
 sta COLUP1
 sta COLUP0
ds_loop

 sta WSYNC
 lda (scoredig0),y ; 5
 sta GRP0 ; 3 = 8
 lda (scoredig0+2),y ; 5 = 13
 sta GRP1 ; 3 = 16
; repeat 16
; nop
; repend ; 48
 ldx #6
ds_delay
 dex
 bne ds_delay ; @ end of loop, 47 cyc.

 lda (scoredig0+4),y ; 5 = 52
 sta GRP0 ; 3 = 55

 sta WSYNC
 lda (scoredig0),y ; 5
 sta GRP0 ; 3 = 8
 lda (scoredig0+2),y ; 5 = 13
 sta GRP1 ; 3 = 16
; repeat 16
; nop
; repend ; 48
 ldx #6
ds_delay1
 dex
 bne ds_delay1 ; @ end of loop, 47 cyc.
 lda (scoredig0+4),y ; 5 = 52
 sta GRP0 ; 3 = 55

 dey

 bpl ds_loop
 sta WSYNC
 lda #0
 sta GRP0
 sta GRP1
 ldx #14
 jmp kill_rest

 echo *-kernel,"bytes of kernel code"
end_kernel
 lda #37
 sta TIM64T

overscan
 lda #2
 sta VBLANK
 
 ldx #$FF
 txs
 lda current_card
 cmp #25
 bne not_score
 jsr calc_score
not_score
 lda playerpos
 bpl finish_overscan
 jsr shuffle

finish_overscan
 lda INTIM
 bne finish_overscan
; sta WSYNC

 jmp main_loop

; suits is 5 elements, each element is the suit (0-4) of the card
; ranks is 13 elements, each elem. is how many of that card we have
calc_score
 lda already_scored
 beq ok_to_score
 rts
ok_to_score
 lda #0
 ldx #28
; stx already_scored
clear_vars
 sta card0ptr,x
 dex
 bpl clear_vars

 ldx score_frame ; switch(score_frame) {
 beq hand_0 ; case 0: { hand_1();break; }
 dex
 beq hand_1 ; case 1:
 dex
 beq hand_2 ; ...
 dex
 beq hand_3
 dex
 beq hand_4
 dex
 beq hand_5
 dex
 beq hand_6
 dex
 bne no_hand_7
 jmp hand_7
no_hand_7
 dex
 bne no_hand_8
 jmp hand_8
no_hand_8
 dex
 bne no_hand_9 ; case 9: { hand_10();break; }
 jmp hand_9
no_hand_9
 ; default:
 stx already_scored
 rts

 mac vert_copy
 ldx #4
.vc_loop
 lda tableau+{1}*5,x
 sta tmp_cards,x
 dex
 bpl .vc_loop
 jmp done_copying
 endm

 mac horiz_copy
 lda tableau+{1}
 sta tmp_cards
 lda tableau+{1}+5
 sta tmp_cards+1
 lda tableau+{1}+10
 sta tmp_cards+2
 lda tableau+{1}+15
 sta tmp_cards+3
 lda tableau+{1}+20
 sta tmp_cards+4
 jmp done_copying
 endm

hand_0
	lda #SND_SCORE
	sta sound_type
	lda #SND_SCORE_LEN
   sta sound_ctr
	vert_copy 0

hand_1 vert_copy 1
hand_2 vert_copy 2
hand_3 vert_copy 3
hand_4 vert_copy 4

hand_5 horiz_copy 0
hand_6 horiz_copy 1
hand_7 horiz_copy 2
hand_8 horiz_copy 3
hand_9 horiz_copy 4

done_copying
 inc score_frame
; this sucks!
 ldx #4
split_loop
 lda tmp_cards,x
sub_loop
 sec
 sbc #13
 bcc no_subtract
 inc suits,x
 bne sub_loop ; should always branch
no_subtract
 adc #13 ; carry should already be clear
 ;tay
 ;inc ranks,y ;arrrgh. illegal addr. mode!
 stx tmp
 tax
 inc ranks,x
 ldx tmp
end_split_loop
 dex
 bpl split_loop

; lda suits+1 ; debug code
; sta score
; rts

; ldx #3
;ck_flush 
; lda suits,x
; cmp #5
; beq got_flush
; dex
; bpl ck_flush
; bmi done_flush


 lda suits
 cmp suits+1
 bne done_flush
 cmp suits+2
 bne done_flush
 cmp suits+3
 bne done_flush
 cmp suits+4
 bne done_flush

got_flush
 inc flush
 bne pre_ck_straight ; if flush, don't bother checking for pairs/trips/etc.
done_flush

 ldx #12
ck_quad
 lda ranks,x
 cmp #4
 bne not_quad
 lda #16 ; if we got 4 of a kind, that's all we got
 jmp add_A_to_score ; so don't check for more stuff
not_quad
 dex
 bpl ck_quad

 ldx #12
ck_trip
 lda ranks,x
 cmp #3
 bne no_trip
 inc trips
 bne done_trips ; if we got trips, we only got it once 
no_trip
 dex
 bpl ck_trip

done_trips
 ldx #12
ck_pair
 lda ranks,x
 cmp #2
 bne no_pair
 inc pairs ; might have 2 pair, keep checking
no_pair
 dex
 bpl ck_pair

pre_ck_straight
 lda ranks
 sta ranks+13 ; duplicate ace as high card
 ldx #9
ck_straight
 lda ranks,x
 and ranks+1,x
 and ranks+2,x
 and ranks+3,x
 and ranks+4,x
 bne got_straight
 dex
 bpl ck_straight
 bmi done_checking
got_straight
 and flush ; did we get a straight flush?
 bne got_straight_flush ; if so, jump
 lda #12 ; otherwise, score 12 for straight
 bne add_A_to_score
got_straight_flush
 lda #30
 bne add_A_to_score

done_checking
; done checking, now total 'em up

 lda flush
 beq test_trips
 lda #5
 bne add_A_to_score

; at this point, we have 1 of:
;  nothing pair 2pair trips fullhouse

test_trips
 lda trips
 beq no_trips
 and pairs ; should be 0 or 1
 bne got_full_house ; if (trips && pairs), we got full house, yay!
 lda #6 ; otherwise, 6 points for trips
 bne add_A_to_score
got_full_house
 lda #10
 bne add_A_to_score
no_trips

; at this point, we have 1 of: nothing pair 2pair


test_pairs
 lda pairs
 beq add_A_to_score
 cmp #2
 beq score_2_pair
 lda #1
 bne add_A_to_score
score_2_pair
 lda #3

add_A_to_score
 clc
 adc score
 sta score

 rts


; tya
; ora #6
; sta.w COLUPF

END_CODE = *
 echo END_CODE-$f000,"bytes of code"

 org $Fa00
 echo *-END_CODE , "bytes of code space free"
 ;include "pokersol.inc"

 ;this playfield data was generated by pfgen.pl, it's not meant
 ;to be read/modified by humans

left_pf0
 byte %01100000
 byte %10010000
 byte %01000000
 byte %00100000
 byte %10010000
 byte %01100000
 byte %00000000
 byte %00000000
 byte %01000000
 byte %01000000
 byte %11110000
 byte %01010000
 byte %01010000
 byte %01000000
 byte %00000000
 byte %00000000
 byte %00010000
 byte %10010000
 byte %10010000
 byte %10110000
 byte %10010000
 byte %01110000
 byte %00000000
 byte %00000000
 byte %00010000
 byte %00010000
 byte %00010000
 byte %00110000
 byte %00010000
 byte %01110000
 byte %00000000
 byte %00000000
 byte %01100000
 byte %10010000
 byte %01000000
 byte %00100000
 byte %10010000
 byte %01100000
 byte %00000000
 byte %00000000
 byte %01110000
 byte %10000000
 byte %10000000
 byte %01100000
 byte %10000000
 byte %01110000
 byte %00000000
 byte %00000000
 byte %11110000
 byte %00010000
 byte %00100000
 byte %01000000
 byte %10000000
 byte %01110000
 byte %00000000
 byte %00000000
 byte %00010000
 byte %00010000
 byte %01110000
 byte %10010000
 byte %10010000
 byte %01110000
pf_bytes = [.-left_pf0]-1


left_pf1
 byte %00010100
 byte %00100000
 byte %00100000
 byte %00100000
 byte %01110000
 byte %00100000
 byte %00000000
 byte %00000000
 byte %00010001
 byte %00101001
 byte %00101001
 byte %00101011
 byte %00010001
 byte %00000000
 byte %00000000
 byte %00000000
 byte %11010100
 byte %01010100
 byte %01010100
 byte %01010100
 byte %01010100
 byte %00010100
 byte %00000000
 byte %00000000
 byte %10011011
 byte %10101000
 byte %10101001
 byte %10101010
 byte %10101001
 byte %10000000
 byte %00000000
 byte %00000000
 byte %00010100
 byte %00100100
 byte %00100100
 byte %00100101
 byte %01110110
 byte %00100000
 byte %00000000
 byte %00000000
 byte %00010001
 byte %00101001
 byte %00101001
 byte %00101011
 byte %00010001
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00100001
 byte %00100001
 byte %00111000
 byte %00100100
 byte %00100101
 byte %00111000
 byte %00000000
 byte %00000000
 byte %01110101
 byte %01010101
 byte %00110101
 byte %00010001
 byte %01100101
 byte %00000000

left_pf2
 byte %10010001
 byte %01010001
 byte %01010001
 byte %01010011
 byte %01010001
 byte %00010111
 byte %00000000
 byte %00000000
 byte %10101000
 byte %10101000
 byte %10101000
 byte %00011001
 byte %10101000
 byte %00101001
 byte %00000000
 byte %00000000
 byte %00100101
 byte %01010101
 byte %01010101
 byte %01010111
 byte %00100101
 byte %00000101
 byte %00000000
 byte %00000000
 byte %00010100
 byte %00010101
 byte %00010100
 byte %00001100
 byte %00000101
 byte %00000100
 byte %00000000
 byte %00000000
 byte %10101110
 byte %00101010
 byte %00101100
 byte %10001000
 byte %00100110
 byte %00000000
 byte %00000000
 byte %00000000
 byte %10101000
 byte %10101000
 byte %10101000
 byte %00011001
 byte %10101000
 byte %00101001
 byte %00000000
 byte %00000000
 byte %00101011
 byte %00101010
 byte %00101011
 byte %10100010
 byte %01101001
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000010
 byte %00000001
 byte %00000000

right_pf0
 byte %11010000
 byte %00010000
 byte %10010000
 byte %01010000
 byte %10010000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %10100000
 byte %10100000
 byte %10100000
 byte %10100000
 byte %01100000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %01100000
 byte %01010000
 byte %01010000
 byte %01010000
 byte %01010000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %10010000
 byte %10100000
 byte %10110000
 byte %10100000
 byte %10010000
 byte %10000000
 byte %00000000
 byte %00000000
 byte %10100000
 byte %10100000
 byte %10100000
 byte %10100000
 byte %01100000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 ;echo *
 org [>.]*256+256 ; 8 bytes a-wasted...
 ;echo *

right_pf1
 byte %00101000
 byte %10101000
 byte %00101000
 byte %00110000
 byte %10100000
 byte %00100000
 byte %00000000
 byte %00000000
 byte %01110000
 byte %01010000
 byte %01010000
 byte %00110000
 byte %00010000
 byte %00010000
 byte %00000000
 byte %00000000
 byte %11000110
 byte %00101000
 byte %01001100
 byte %10001010
 byte %01100100
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %01001000
 byte %01010000
 byte %01010000
 byte %10010000
 byte %00111000
 byte %00010000
 byte %00000000
 byte %00000000
 byte %01110000
 byte %01010000
 byte %01010000
 byte %00110000
 byte %00010000
 byte %00010000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000
 byte %00000000

right_pf2
 byte %01100011
 byte %10010100
 byte %10010100
 byte %10010010
 byte %10010100
 byte %01100011
 byte %00000000
 byte %00000000
 byte %01100111
 byte %10010010
 byte %10010010
 byte %01110010
 byte %00010011
 byte %11100010
 byte %00000000
 byte %00000000
 byte %01100111
 byte %10010010
 byte %10010010
 byte %10010010
 byte %10010011
 byte %01100010
 byte %00000000
 byte %00000000
 byte %01100000
 byte %10010000
 byte %10000000
 byte %01110000
 byte %00010000
 byte %11110000
 byte %00000000
 byte %00000000
 byte %11110111
 byte %00010010
 byte %00100010
 byte %01000010
 byte %10000011
 byte %01110010
 byte %00000000
 byte %00000000
 byte %01100000
 byte %10010000
 byte %10010000
 byte %01110000
 byte %00010000
 byte %11100000
 byte %00000000
 byte %00000000
 byte %01110000
 byte %10000000
 byte %10000000
 byte %01100000
 byte %10000000
 byte %01110000
 byte %00000000
 byte %00000000
 byte %11100000
 byte %01000000
 byte %01000000
 byte %01000000
 byte %01100000
 byte %01000000

; SUBROUTINES
;
; these were moved here because there was a small (132-byte) chunk of space
; wasted by the next `align 256'. There are still 5 bytes of this left.

inst_kernel ; this kernel displays the instructions
 ldy #pf_bytes
 lda framectr
 lsr
 lsr
 lsr
 ora #8
 and bw_mask

 sta COLUPF
ik_draw_pf
 lda #2
 sta tmp
ik_one_line
 sta WSYNC
 lda left_pf0,y
 sta PF0
 lda left_pf1,y
 sta PF1
 lda left_pf2,y
 sta PF2
 lda right_pf1,y
 ldx right_pf2,y
 nop
 nop
 nop
 nop
 sta PF1
 lda right_pf0,y
 sta PF0
 stx PF2
 dec tmp
 bpl ik_one_line
 dey
 bpl ik_draw_pf

 sta WSYNC
 lda #0
 sta COLUPF
 repeat 4
 sta WSYNC
 repend

 jmp end_kernel

deal_deck
 ldx #0
 ldy #0
 stx current_card
 stx already_scored
 stx score_frame
 stx score
deal_loop
 stx deck,y
 iny
 inx
 cpy #52
 bne deal_loop 
 rts

; shuffle deck
shuffle
 ldy #50
shuffle_loop
 lda random ; 3
 lsr ; +2 = 5
 lsr ; +2 = 7
 sbc random ; +3 = 10
 lsr ; +2 = 12
 ror random+1 ; +5 = 17
 ror random ; +5 = 17
 bcs noswap ; +3 = 20
 lda deck,y ; +4 = 24 ; if we got a 1 (carry), swap card with the one after it
 ldx deck+1,y ; +4 = 28
 stx deck,y ; +4 = 32
 sta deck+1,y ; +4 = 38
noswap
 dey ; +2 = 40
 bpl shuffle_loop ; +3 = 43
 rts

END_PF = *

 align 256
 echo *-END_PF, "bytes wasted in data area: ", END_PF, "-", *
score_font
sf0
 byte %00111100
 byte %01111110
 byte %11100111
 byte %11000011
 byte %11000011
 byte %11100111
 byte %01111110
 byte %00111100
sf1
 byte %01111110
 byte %01111110
 byte %00011000
 byte %00011000
 byte %00011000
 byte %01111000
 byte %00111000
 byte %00011000
sf2
 byte %11111111
 byte %01111111
 byte %00110000
 byte %00011100
 byte %00000110
 byte %01100011
 byte %11111111
 byte %01111110
sf3
; byte %00111000
; byte %11111110
; byte %11000111
; byte %00011110
; byte %00011100
; byte %11001110
; byte %11111100
; byte %00111000
 byte %01111110
 byte %11111111
 byte %01100011
 byte %00000110
 byte %00011100
 byte %00000110
 byte %00000011
 byte %11111111
sf4
 byte %00000110
 byte %00000110
 byte %11111111
 byte %11111111
 byte %11000110
 byte %11100110
 byte %01110110
 byte %00110110
sf5
 byte %01111100
 byte %11111110
 byte %01100111
 byte %00001110
 byte %11111100
 byte %11000000
 byte %11111111
 byte %11111111
sf6
 byte %01111100
 byte %11111110
 byte %11001110
 byte %11011100
 byte %11111000
 byte %11100011
 byte %01111111
 byte %00111110
sf7
 byte %11100000
 byte %01110000
 byte %00111000
 byte %00011100
 byte %00001110
 byte %00000111
 byte %11111111
 byte %11111111
sf8
 byte %01111110
 byte %11111111
 byte %11100111
 byte %01111110
 byte %01111110
 byte %11100111
 byte %11111111
 byte %01111110
sf9
 byte %01111100
 byte %11111110
 byte %11000111
 byte %00011111
 byte %00111011
 byte %01110011
 byte %01111111
 byte %00111110
; byte %11100000
; byte %01110000
; byte %00111000
; byte %00011100
; byte %01111110
; byte %11100111
; byte %01111110
; byte %00111100
score_blank
no_card
 repeat 14
 byte 0
 repend 

;stacked ; used only in debugging
; byte 36,6,10,49,11

colPF0
 repeat 16
 byte $f0
 repend
 repeat 16
 byte $c0
 repend

colPF1
 repeat 4
 byte $ff,$ff,$ff,$ff,$f3,$f3,$f3,$f3,$3f,$3f,$3f,$3f,$33,$33,$33,$33
 repend

colPF2
 repeat 8
 byte $ff,$cf,$fc,$cc
 repend

; jumped to by reset vector (crammed in here to fill up $22 byte wasted space)

initvcs
 sei
 cld
 ldx #$ff
 txs
 lda #0
iloop
 sta 0,x
 dex
 bne iloop
 lda #55
 sta random
 lda #$AA
 sta random+1
 lda #$FF
 sta playerpos

; initialize deck
 jsr deal_deck
 jmp main_loop

END_MISC = *
 ;org $Fc00
 align 256
 echo *-END_MISC, "bytes wasted in MISC data area: ", END_MISC, "-", *
cards
spades1
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %10111011
 byte %10111011
 byte %10000011
 byte %11010111
 byte %11101111
spades2
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %10000011
 byte %11011111
 byte %11100111
 byte %10111011
 byte %11000111
spades3
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %10000111
 byte %11111011
 byte %11100111
 byte %10111011
 byte %11000111
spades4
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %11111011
 byte %10000011
 byte %10111011
 byte %11011011
 byte %11101011
spades5
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %11000111
 byte %10111011
 byte %11100111
 byte %11011111
 byte %11000011
spades6
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %11000111
 byte %10111011
 byte %10000111
 byte %10111111
 byte %11000011
spades7
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %11011111
 byte %11101111
 byte %11110111
 byte %11111011
 byte %10000011
spades8
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %11000111
 byte %10111011
 byte %11000111
 byte %10111011
 byte %11000111
spades9
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %10000111
 byte %11111011
 byte %11000011
 byte %10111011
 byte %11000111
spades10
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %00010011
 byte %10101101
 byte %10101101
 byte %00101101
 byte %10110011
spadesj
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %11001111
 byte %10110111
 byte %11110111
 byte %11110111
 byte %11100011
spadesq
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %11111011
 byte %11000111
 byte %10111011
 byte %10111011
 byte %11000111
spadesk
 byte %11000111
 byte %01101101
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111
 byte %11111111
 byte %10110111
 byte %10101111
 byte %10011111
 byte %10101111
 byte %10110111
clubs1
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111
 byte %11111111
 byte %10111011
 byte %10111011
 byte %10000011
 byte %11010111
 byte %11101111
clubs2
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111

 byte %11111111
 byte %10000011
 byte %11011111
 byte %11100111
 byte %10111011
 byte %11000111
clubs3
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111

 byte %11111111
 byte %10000111
 byte %11111011
 byte %11100111
 byte %10111011
 byte %11000111
clubs4
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111

 byte %11111111
 byte %11111011
 byte %10000011
 byte %10111011
 byte %11011011
 byte %11101011
clubs5
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111

 byte %11111111
 byte %11000111
 byte %10111011
 byte %11100111
 byte %11011111
 byte %11000011
clubs6
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111

 byte %11111111
 byte %11000111
 byte %10111011
 byte %10000111
 byte %10111111
 byte %11000011
clubs7
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111

 byte %11111111
 byte %11011111
 byte %11101111
 byte %11110111
 byte %11111011
 byte %10000011
clubs8
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111

 byte %11111111
 byte %11000111
 byte %10111011
 byte %11000111
 byte %10111011
 byte %11000111
clubs9
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111

 byte %11111111
 byte %10000111
 byte %11111011
 byte %11000011
 byte %10111011
 byte %11000111
clubs10
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111

 byte %11111111
 byte %00010011
 byte %10101101
 byte %10101101
 byte %00101101
 byte %10110011
clubsj
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111

 byte %11111111
 byte %11001111
 byte %10110111
 byte %11110111
 byte %11110111
 byte %11100011
clubsq
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111

 byte %11111111
 byte %11111011
 byte %11000111
 byte %10111011
 byte %10111011
 byte %11000111
clubsk
 byte %11000111
 byte %11101111
 byte %00101001
 byte %00111001
 byte %11000111
 byte %11000111

 byte %11111111
 byte %10110111
 byte %10101111
 byte %10011111
 byte %10101111
 byte %10110111
hearts1
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011

 byte %11111111
 byte %10111011
 byte %10111011
 byte %10000011
 byte %11010111
 byte %11101111
hearts2
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011

 byte %11111111
 byte %10000011
 byte %11011111
 byte %11100111
 byte %10111011
 byte %11000111
hearts3
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011

 byte %11111111
 byte %10000111
 byte %11111011
 byte %11100111
 byte %10111011
 byte %11000111
hearts4
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011

 byte %11111111
 byte %11111011
 byte %10000011
 byte %10111011
 byte %11011011
 byte %11101011
hearts5
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011

 byte %11111111
 byte %11000111
 byte %10111011
 byte %11100111
 byte %11011111
 byte %11000011
hearts6
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011

 byte %11111111
 byte %11000111
 byte %10111011
 byte %10000111
 byte %10111111
 byte %11000011
hearts7
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011


 byte %11111111
 byte %11011111
 byte %11101111
 byte %11110111
 byte %11111011
 byte %10000011
hearts8
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011


 byte %11111111
 byte %11000111
 byte %10111011
 byte %11000111
 byte %10111011
 byte %11000111
hearts9
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011


 byte %11111111
 byte %10000111
 byte %11111011
 byte %11000011
 byte %10111011
 byte %11000111
hearts10
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011


 byte %11111111
 byte %00010011
 byte %10101101
 byte %10101101
 byte %00101101
 byte %10110011
heartsj
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011


 byte %11111111
 byte %11001111
 byte %10110111
 byte %11110111
 byte %11110111
 byte %11100011
heartsq
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011


 byte %11111111
 byte %11111011
 byte %11000111
 byte %10111011
 byte %10111011
 byte %11000111
heartsk
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %00010001
 byte %10111011


 byte %11111111
 byte %10110111
 byte %10101111
 byte %10011111
 byte %10101111
 byte %10110111
 byte %01111110
diam1
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %10111011
 byte %10111011
 byte %10000011
 byte %11010111
 byte %11101111
diam2
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %10000011
 byte %11011111
 byte %11100111
 byte %10111011
 byte %11000111
diam3
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %10000111
 byte %11111011
 byte %11100111
 byte %10111011
 byte %11000111
diam4
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %11111011
 byte %10000011
 byte %10111011
 byte %11011011
 byte %11101011
diam5
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %11000111
 byte %10111011
 byte %11100111
 byte %11011111
 byte %11000011
diam6
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %11000111
 byte %10111011
 byte %10000111
 byte %10111111
 byte %11000011
diam7
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %11011111
 byte %11101111
 byte %11110111
 byte %11111011
 byte %10000011
diam8
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %11000111
 byte %10111011
 byte %11000111
 byte %10111011
 byte %11000111
diam9
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %10000111
 byte %11111011
 byte %11000011
 byte %10111011
 byte %11000111
diam10
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %00010011
 byte %10101101
 byte %10101101
 byte %00101101
 byte %10110011
diamj
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %11001111
 byte %10110111
 byte %11110111
 byte %11110111
 byte %11100011
diamq
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %11111011
 byte %11000111
 byte %10111011
 byte %10111011
 byte %11000111
diamk
 byte %11101111
 byte %11000111
 byte %10000011
 byte %00000001
 byte %10000011
 byte %11000111
 byte %11101111

 byte %10110111
 byte %10101111
 byte %10011111
 byte %10101111
 byte %10110111
blank_card
 byte %11111111
 byte %11111111
 byte %11111111
 byte %11111111
 byte %11111111
 byte %11111111
 byte %11111111
 byte %11111111
 byte %11111111
 byte %11111111
 byte %11111111
 byte %11111111
;no_card
; repeat 14
; byte 0
; repend

lo_bytes
 byte <spades1, <spades2, <spades3, <spades4, <spades5
 byte <spades6, <spades7, <spades8, <spades9, <spades10
 byte <spadesj, <spadesq, <spadesk
 byte <clubs1, <clubs2, <clubs3, <clubs4, <clubs5
 byte <clubs6, <clubs7, <clubs8, <clubs9, <clubs10
 byte <clubsj, <clubsq, <clubsk
 byte <hearts1, <hearts2, <hearts3, <hearts4, <hearts5
 byte <hearts6, <hearts7, <hearts8, <hearts9, <hearts10
 byte <heartsj, <heartsq, <heartsk
 byte <diam1, <diam2, <diam3, <diam4, <diam5
 byte <diam6, <diam7, <diam8, <diam9, <diam10
 byte <diamj, <diamq, <diamk, <blank_card, <no_card
hi_bytes
 byte >spades1, >spades2, >spades3, >spades4, >spades5
 byte >spades6, >spades7, >spades8, >spades9, >spades10
 byte >spadesj, >spadesq, >spadesk
 byte >clubs1, >clubs2, >clubs3, >clubs4, >clubs5
 byte >clubs6, >clubs7, >clubs8, >clubs9, >clubs10
 byte >clubsj, >clubsq, >clubsk
 byte >hearts1, >hearts2, >hearts3, >hearts4, >hearts5
 byte >hearts6, >hearts7, >hearts8, >hearts9, >hearts10
 byte >heartsj, >heartsq, >heartsk
 byte >diam1, >diam2, >diam3, >diam4, >diam5
 byte >diam6, >diam7, >diam8, >diam9, >diam10
 byte >diamj, >diamq, >diamk, >blank_card, >no_card

; colors:
;

; PF0: %aaaaXXXX  PF1: %bbbbcccc %PF2 %ddddeeee

; a = column 0 ($c = red, $f = black)
; b = column 1 ($3 = red, $f = black)
; c = column 2 ($3 = red, $f = black)
; d = column 4 ($c = red, $f = black)
; e = column 3 ($c = red, $f = black)

; echo $ff00-*, "bytes wasted in data area at", *
; org $FF00 ; waste a couple bytes, make sure tables don't cross page.

 ;echo $10000-*, "bytes wasted in data area at", *

; left & right diff. graphics

left_diff_hard
 byte %00000000
 byte %11000011
 byte %11000011
 byte %00011000
 byte %00011000
 byte %11000011
 byte %11000011
 byte %00000000
right_diff_hard
 byte %10000010
 byte %01000100
 byte %00101000
 byte %00010000 
 byte %00101000
 byte %01000100
 byte %10000010
 byte %00000000

END_DATA = *

 org $FFFC
 echo *-END_DATA, "bytes wasted in data area: ", END_DATA, "-", *
 word initvcs
 word initvcs

 echo *-left_pf0,"bytes of data"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; TODO LIST
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; STARTED ON:
; optimize, optimize, optimize! (specifically, get rid of macros & repeats)
; I now have 482 bytes free in the code area from $F000-$FA00

; possibly center the cards on the display, ref. Manuel's post 11/22/01

; play testing! (by someone other than me!)

; STARTED ON: get rid of `magic numbers', make them constants

; STARTED ON: Clean up score font some (3, 5, 8 at least)

;;;;;
;
; Stuff that was on the TODO list that is mostly or all done:
;
;;;;;

; (mostly) DONE: Sound

; DONE: Help/Scoring screen (SELECT)

; (I think) DONE: Figure out what caused the glitch (screenshot: graphics_glitch.jpg)

; DONE: joystick debounce

; DONE: Make sure calc_score doesn't eat too many cycles (just test on real hardware
; & see if it glitches)

; DONE: Blank leading zeroes in score

; DONE:
; Decide whether royal flush should be 36 points or 30 like other
; straight flushes (Hoyle says 30, so do online rules, but I like 36... Also
; the best 5x5 grid would then be worth 254 points!).. I haven't decided
; whether or not to do this, but I have decided to hard-code it instead of
; making it optional (so people can compare their scores without having to
; take into account how the scoring was done).
;
; I've decided not to bother with 36-point royal flushes. It doesn't really
; add anything to the game, and adds cycles to the calc_score routine.

; DONE: color the score? (red=below 25, yellow=25 to 40, grn=40 & up?)

; DONE: Make left difficulty actually make the game harder

; DONE: Add some kind of indicator to let the player know whether he's about to play
; the hard or easy game (H or E in 4th card column?)

; DONE: implement color/bw switch

;;;;;
;
; Stuff I may do, if I can free up some more RAM/ROM:
;
;;;;;

; Make right difficulty control whether diagonals are scored or not (only
; poll during the shuffle)

; add an indicator to let the player know whether diags. will be scored.

; perhaps slow down scoring & flash the row/col being scored?

; 2-player mode? (all we really have to do is score vert & horiz separately)
